@@ -2,9 +2,9 @@
 #
 #  Net::Server::INET - Net::Server personality
 #
-#  $Id: INET.pm,v 1.6 2005/06/20 18:48:50 rhandom Exp $
+#  $Id: INET.pm,v 1.8 2010/05/05 04:38:57 rhandom Exp $
 #
-#  Copyright (C) 2001-2005
+#  Copyright (C) 2001-2007
 #
 #    Paul Seamons
 #    paul@seamons.com
@@ -176,6 +176,26 @@ sub DESTROY {}
 sub FILENO {}
 
 sub FETCH {}
+
+sub read_until { # only sips the data - but it allows for compatibility with SSLEAY
+    my ($client, $bytes, $end_qr) = @_;
+    die "One of bytes or end_qr should be defined for TCP read_until\n" if !defined($bytes) && !defined($end_qr);
+    my $content = '';
+    my $ok = 0;
+    while (1) {
+        $client->read($content, 1, length($content));
+        if (defined($bytes) && length($content) >= $bytes) {
+            $ok = 2;
+            last;
+        }
+        elsif (defined($end_qr) && $content =~ $end_qr) {
+            $ok = 1;
+            last;
+        }
+    }
+    return wantarray ? ($ok, $content) : $content;
+}
+
 1;
 
 
